/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.collaboration;

import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.NullValueCheckStrategy;
import org.mapstruct.ReportingPolicy;

import de.ozgcloud.collaboration.request.GrpcCollaborationRequest;

@Mapper(unmappedTargetPolicy = ReportingPolicy.ERROR, nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
interface CollaborationMapper {

	default Collaboration fromCollaborationRequest(GrpcCollaborationRequest request) {
		return isFachstelle(request) ? fromFachstelleRequest(request) : fromOrganisationsEinheitRequest(request);
	}

	default boolean isFachstelle(GrpcCollaborationRequest request) {
		return Long.valueOf(4).equals(request.getCollaborationLevel());
	}

	@Mapping(target = "vorgangId", ignore = true)
	OrganisationsEinheitCollaboration fromOrganisationsEinheitRequest(GrpcCollaborationRequest request);

	@Mapping(target = "vorgangId", ignore = true)
	FachstelleCollaboration fromFachstelleRequest(GrpcCollaborationRequest request);
}
