/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.errorhandling;

import java.util.UUID;

import de.ozgcloud.common.errorhandling.ExceptionUtil;
import de.ozgcloud.common.errorhandling.FunctionalErrorCode;
import de.ozgcloud.common.errorhandling.IdentifiableException;

public class FunctionalException extends RuntimeException implements IdentifiableException {

	private static final long serialVersionUID = 1L;

	private final FunctionalErrorCode errorCode;
	private final String exceptionId;

	public FunctionalException(FunctionalErrorCode errorCode) {
		super("Functional error: " + errorCode.getErrorCode());

		this.errorCode = errorCode;
		this.exceptionId = UUID.randomUUID().toString();
	}

	public String getErrorCode() {
		return errorCode.getErrorCode();
	}

	@Override
	public String getMessage() {
		return ExceptionUtil.formatMessageWithExceptionId(super.getMessage(), exceptionId);
	}

	@Override
	public String getExceptionId() {
		return exceptionId;
	}
}