/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import com.fasterxml.jackson.databind.BeanProperty;

class IdBuilderTest {

	@DisplayName("Test building ID when deserializing linked resources")
	@Nested
	class TestBuilingId {
		private static final String ID = "id";

		@Test
		void shouldBuildId() {
			IdBuilder idBuilder = new IdBuilder();

			var idObject = idBuilder.build(ID);

			assertThat(idObject).isInstanceOf(Object.class).asString().isEqualTo(ID);
		}

		@Test
		void shouldCreateObjectBuilder() {
			BeanProperty property = mock(BeanProperty.class);
			ObjectBuilder<Object> idBuilder = new IdBuilder().constructContextAware(property);

			assertThat(idBuilder).isNotNull();
		}
	}
}
