/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.binaryfile;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.concurrent.CompletableFuture;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.test.util.ReflectionTestUtils;

import de.ozgcloud.alfa.common.file.OzgFileTestFactory;
import de.ozgcloud.vorgang.grpc.binaryFile.GrpcUploadBinaryFileResponse;

class BinaryFileUploadStreamObserverTest {

	@InjectMocks
	private BinaryFileUploadStreamObserver uploadStreamObserver;
	@Mock
	private CompletableFuture<FileId> fileIdFuture;

	private final String FIELD_FILE_ID = "fileId";

	@Nested
	class TestOnNext {

		private GrpcUploadBinaryFileResponse uploadResponse = GrpcUploadBinaryFileResponse.newBuilder()
				.setFileId(OzgFileTestFactory.ID.toString())
				.build();

		@Test
		void shouldSetFileId() {
			uploadStreamObserver.onNext(uploadResponse);

			assertThat(uploadStreamObserver.getFileId()).isEqualTo(OzgFileTestFactory.ID.toString());
		}
	}

	@Nested
	class TestOnError {

		@Mock
		private Throwable throwable;

		@Test
		void shouldCompleteFutureExceptionally() {
			uploadStreamObserver.onError(throwable);

			verify(fileIdFuture).completeExceptionally(throwable);
		}
	}

	@Nested
	class TestOnCompleted {

		@Test
		void shouldCompleteWithFileId() {
			ReflectionTestUtils.setField(uploadStreamObserver, FIELD_FILE_ID, BinaryFileTestFactory.ID);

			uploadStreamObserver.onCompleted();

			verify(fileIdFuture).complete(FileId.from(BinaryFileTestFactory.ID));
		}

	}
}
