/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.binaryfile;

import static org.assertj.core.api.Assertions.*;

import java.util.UUID;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;

class FileIdMapperTest {

	private final FileIdMapper mapper = Mappers.getMapper(FileIdMapper.class);
	private final String fileIdValue = UUID.randomUUID().toString();

	@Nested
	class TestToFileId {

		@Test
		void shouldReturnFileId() {
			var fileId = mapper.toFileId(fileIdValue);

			assertThat(fileId).isEqualTo(FileId.from(fileIdValue));
		}
	}

	@Nested
	class TestToString {

		@Test
		void shouldReturnFileIdValue() {
			var fileIdString = mapper.toString(FileId.from(fileIdValue));

			assertThat(fileIdString).isEqualTo(fileIdValue);
		}
	}
}
