/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.forwarding;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.alfa.common.callcontext.ContextService;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.vorgang.forwarding.ForwardingServiceGrpc.ForwardingServiceBlockingStub;
import de.ozgcloud.vorgang.forwarding.GrpcFindForwardingsRequest;
import de.ozgcloud.vorgang.forwarding.GrpcForwarding;
import de.ozgcloud.vorgang.forwarding.GrpcForwardingsResponse;
import de.ozgcloud.vorgang.grpc.command.GrpcCallContext;

class ForwardingRemoteServiceTest {

	@InjectMocks // NOSONAR
	private ForwardingRemoteService service;

	@Mock
	private ForwardingServiceBlockingStub serviceStub;
	@Mock
	private ForwardingMapper mapper;

	@Mock
	private ContextService contextService;

	@Captor
	private ArgumentCaptor<GrpcFindForwardingsRequest> requestCaptor;

	private GrpcCallContext callContext = GrpcCallContext.newBuilder().build();

	@BeforeEach
	void initContextService() {
		when(contextService.createCallContext()).thenReturn(callContext);
	}

	@Nested
	class TestFindForwardings {
		private GrpcForwarding grpcForwarding = GrpcForwarding.newBuilder().build();
		private GrpcForwardingsResponse response = GrpcForwardingsResponse.newBuilder().addForwardings(grpcForwarding).build();

		@BeforeEach
		void initStub() {
			when(serviceStub.findForwardings(any())).thenReturn(response);
		}

		@Test
		void shouldCallStub() {
			service.findForwardings(VorgangHeaderTestFactory.ID);

			verify(serviceStub).findForwardings(any());
		}

		@Test
		void shouldFillCallContext() {
			service.findForwardings(VorgangHeaderTestFactory.ID);

			verify(serviceStub).findForwardings(requestCaptor.capture());
			assertThat(requestCaptor.getValue().getContext()).isSameAs(callContext);
		}

		@Test
		void shouldHaveVorgangId() {
			service.findForwardings(VorgangHeaderTestFactory.ID);

			verify(serviceStub).findForwardings(requestCaptor.capture());
			assertThat(requestCaptor.getValue().getVorgangId()).isEqualTo(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldCallMapper() {
			service.findForwardings(VorgangHeaderTestFactory.ID).forEach(v -> {
			});

			verify(mapper).fromGrpc(grpcForwarding);
		}
	}

}
